@@ -3,7 +3,7 @@
  *  Definition of optimizing macros. */
 
 /*
- *  GiNaC Copyright (C) 1999-2016 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2022 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,14 +26,11 @@
 #ifdef __GNUC__
 #define unlikely(cond) __builtin_expect((cond), 0)
 #define likely(cond) __builtin_expect((cond), 1)
+#define attribute_deprecated __attribute__ ((deprecated))
 #else
 #define unlikely(cond) (cond)
 #define likely(cond) (cond)
-#endif
-
-#ifdef _MSC_VER
-#define __func__ __FUNCTION__
-#define __alignof__ __alignof
+#define attribute_deprecated
 #endif
 
 #endif // ndef GINAC_COMPILER_DEP_H
