@@ -3,7 +3,7 @@
  *  Interface to GiNaC's light-weight expression handles. */
 
 /*
- *  GiNaC Copyright (C) 1999-2008 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2022 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,29 +20,35 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_EX_H__
-#define __GINAC_EX_H__
-
-#include <iosfwd>
-#include <iterator>
-#include <functional>
-#include <stack>
+#ifndef GINAC_EX_H
+#define GINAC_EX_H
 
 #include "basic.h"
 #include "ptr.h"
 
-namespace GiNaC {
+#include <functional>
+#include <iosfwd>
+#include <iterator>
+#include <memory>
+#include <stack>
 
+namespace GiNaC {
+#ifdef _MSC_VER
+  // MSVC produces a different symbol for _ex0 when it is declared inside   
+  // ex::is_zero() than when it is declared at top level as follows
+  extern const ex _ex0;
+#endif
 
 /** Helper class to initialize the library.  There must be one static object
  *  of this class in every object file that makes use of our flyweights in
  *  order to guarantee proper initialization.  Hence we put it into this
  *  file which is included by every relevant file anyways.  This is modeled
- *  after section 27.4.2.1.6 of the C++ standard, where cout and friends are
+ *  after section [ios::Init] of the C++ standard, where cout and friends are
  *  set up.
  *
  *  @see utils.cpp */
 class library_init {
+	static void init_unarchivers();
 public:
 	library_init();
 	~library_init();
@@ -52,13 +58,6 @@ private:
 /** For construction of flyweights, etc. */
 static library_init library_initializer;
 
-/** Rotate bits of unsigned value by one bit to the left.
-  * This can be necesary if the user wants to define its own hashes. */
-inline unsigned rotate_left(unsigned n)
-{
-	return (n & 0x80000000U) ? (n << 1 | 0x00000001U) : (n << 1);
-}
-
 class scalar_products;
 class const_iterator;
 class const_preorder_iterator;
@@ -79,7 +78,7 @@ class ex {
 	
 	// default constructor, copy constructor and assignment operator
 public:
-	ex() throw();
+	ex() noexcept;
 
 	// other constructors
 public:
@@ -88,6 +87,8 @@ public:
 	ex(unsigned int i);
 	ex(long i);
 	ex(unsigned long i);
+	ex(long long i);
+	ex(unsigned long long i);
 	ex(double const d);
 
 	/** Construct ex from string and a list of symbols. The input grammar is
@@ -100,7 +101,7 @@ public:
 	// non-virtual functions in this class
 public:
 	/** Efficiently swap the contents of two expressions. */
-	void swap(ex & other) throw()
+	void swap(ex & other) noexcept
 	{
 		GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 		GINAC_ASSERT(other.bp->flags & status_flags::dynallocated);
@@ -108,16 +109,16 @@ public:
 	}
 
 	// iterators
-	const_iterator begin() const throw();
-	const_iterator end() const throw();
+	const_iterator begin() const noexcept;
+	const_iterator end() const noexcept;
 	const_preorder_iterator preorder_begin() const;
-	const_preorder_iterator preorder_end() const throw();
+	const_preorder_iterator preorder_end() const noexcept;
 	const_postorder_iterator postorder_begin() const;
-	const_postorder_iterator postorder_end() const throw();
+	const_postorder_iterator postorder_end() const noexcept;
 
 	// evaluation
-	ex eval(int level = 0) const { return bp->eval(level); }
-	ex evalf(int level = 0) const { return bp->evalf(level); }
+	ex eval() const { return bp->eval(); }
+	ex evalf() const { return bp->evalf(); }
 	ex evalm() const { return bp->evalm(); }
 	ex eval_ncmul(const exvector & v) const { return bp->eval_ncmul(v); }
 	ex eval_integ() const { return bp->eval_integ(); }
@@ -133,8 +134,8 @@ public:
 	// operand access
 	size_t nops() const { return bp->nops(); }
 	ex op(size_t i) const { return bp->op(i); }
-	ex operator[](const ex & index) const { return (*bp)[index]; }
-	ex operator[](size_t i) const { return (*bp)[i]; }
+	ex operator[](const ex & index) const { return (const_cast<const basic&>(*bp))[index]; }
+	ex operator[](size_t i) const { return (const_cast<const basic&>(*bp))[i]; }
 	ex & let_op(size_t i);
 	ex & operator[](const ex & index);
 	ex & operator[](size_t i);
@@ -148,9 +149,9 @@ public:
 
 	// pattern matching
 	bool has(const ex & pattern, unsigned options = 0) const { return bp->has(pattern, options); }
-	bool find(const ex & pattern, lst & found) const;
+	bool find(const ex & pattern, exset& found) const;
 	bool match(const ex & pattern) const;
-	bool match(const ex & pattern, lst & repl_lst) const { return bp->match(pattern, repl_lst); }
+	bool match(const ex & pattern, exmap & repls) const { return bp->match(pattern, repls); }
 
 	// substitutions
 	ex subs(const exmap & m, unsigned options = 0) const;
@@ -184,11 +185,9 @@ public:
 	ex series(const ex & r, int order, unsigned options = 0) const;
 
 	// rational functions
-	ex normal(int level = 0) const;
+	ex normal() const;
 	ex to_rational(exmap & repl) const;
-	ex to_rational(lst & repl_lst) const;
 	ex to_polynomial(exmap & repl) const;
-	ex to_polynomial(lst & repl_lst) const;
 	ex numer() const;
 	ex denom() const;
 	ex numer_denom() const;
@@ -211,7 +210,12 @@ public:
 	// comparison
 	int compare(const ex & other) const;
 	bool is_equal(const ex & other) const;
-	bool is_zero() const { extern const ex _ex0; return is_equal(_ex0); }
+	bool is_zero() const { 
+#ifndef _MSC_VER
+	  extern const ex _ex0;
+#endif
+	  return is_equal(_ex0); 
+	}
 	bool is_zero_matrix() const;
 	
 	// symmetry
@@ -224,7 +228,7 @@ public:
 
 	// noncommutativity
 	unsigned return_type() const { return bp->return_type(); }
-	tinfo_t return_type_tinfo() const { return bp->return_type_tinfo(); }
+	return_type_t return_type_tinfo() const { return bp->return_type_tinfo(); }
 
 	unsigned gethash() const { return bp->gethash(); }
 
@@ -234,6 +238,8 @@ private:
 	static basic & construct_from_uint(unsigned int i);
 	static basic & construct_from_long(long i);
 	static basic & construct_from_ulong(unsigned long i);
+	static basic & construct_from_longlong(long long i);
+	static basic & construct_from_ulonglong(unsigned long long i);
 	static basic & construct_from_double(double d);
 	static ptr<basic> construct_from_string_and_lst(const std::string &s, const ex &l);
 	void makewriteable();
@@ -253,7 +259,7 @@ private:
 extern const basic *_num0_bp;
 
 inline
-ex::ex() throw() : bp(*const_cast<basic *>(_num0_bp))
+ex::ex() noexcept : bp(*const_cast<basic *>(_num0_bp))
 {
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
@@ -288,6 +294,18 @@ ex::ex(unsigned long i) : bp(construct_from_ulong(i))
 	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
 }
 
+inline
+ex::ex(long long i) : bp(construct_from_longlong(i))
+{
+	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
+}
+
+inline
+ex::ex(unsigned long long i) : bp(construct_from_ulonglong(i))
+{
+	GINAC_ASSERT(bp->flags & status_flags::dynallocated);
+}
+
 inline
 ex::ex(double const d) : bp(construct_from_double(d))
 {
@@ -355,10 +373,10 @@ class const_iterator : public std::iterator<std::random_access_iterator_tag, ex,
 	friend class const_postorder_iterator;
 
 public:
-	const_iterator() throw() {}
+	const_iterator() noexcept {}
 
 private:
-	const_iterator(const ex &e_, size_t i_) throw() : e(e_), i(i_) {}
+	const_iterator(const ex &e_, size_t i_) noexcept : e(e_), i(i_) {}
 
 public:
 	// This should return an ex&, but that would be a reference to a
@@ -370,9 +388,9 @@ public:
 
 	// This should return an ex*, but that would be a pointer to a
 	// temporary value
-	std::auto_ptr<ex> operator->() const
+	std::unique_ptr<ex> operator->() const
 	{
-		return std::auto_ptr<ex>(new ex(operator*()));
+		return std::unique_ptr<ex>(new ex(operator*()));
 	}
 
 	ex operator[](difference_type n) const
@@ -380,90 +398,90 @@ public:
 		return e.op(i + n);
 	}
 
-	const_iterator &operator++() throw()
+	const_iterator &operator++() noexcept
 	{
 		++i;
 		return *this;
 	}
 
-	const_iterator operator++(int) throw()
+	const_iterator operator++(int) noexcept
 	{
 		const_iterator tmp = *this;
 		++i;
 		return tmp;
 	}
 
-	const_iterator &operator+=(difference_type n) throw()
+	const_iterator &operator+=(difference_type n) noexcept
 	{
 		i += n;
 		return *this;
 	}
 
-	const_iterator operator+(difference_type n) const throw()
+	const_iterator operator+(difference_type n) const noexcept
 	{
 		return const_iterator(e, i + n);
 	}
 
-	inline friend const_iterator operator+(difference_type n, const const_iterator &it) throw()
+	inline friend const_iterator operator+(difference_type n, const const_iterator &it) noexcept
 	{
 		return const_iterator(it.e, it.i + n);
 	}
 
-	const_iterator &operator--() throw()
+	const_iterator &operator--() noexcept
 	{
 		--i;
 		return *this;
 	}
 
-	const_iterator operator--(int) throw()
+	const_iterator operator--(int) noexcept
 	{
 		const_iterator tmp = *this;
 		--i;
 		return tmp;
 	}
 
-	const_iterator &operator-=(difference_type n) throw()
+	const_iterator &operator-=(difference_type n) noexcept
 	{
 		i -= n;
 		return *this;
 	}
 
-	const_iterator operator-(difference_type n) const throw()
+	const_iterator operator-(difference_type n) const noexcept
 	{
 		return const_iterator(e, i - n);
 	}
 
-	inline friend difference_type operator-(const const_iterator &lhs, const const_iterator &rhs) throw()
+	inline friend difference_type operator-(const const_iterator &lhs, const const_iterator &rhs) noexcept
 	{
 		return lhs.i - rhs.i;
 	}
 
-	bool operator==(const const_iterator &other) const throw()
+	bool operator==(const const_iterator &other) const noexcept
 	{
 		return are_ex_trivially_equal(e, other.e) && i == other.i;
 	}
 
-	bool operator!=(const const_iterator &other) const throw()
+	bool operator!=(const const_iterator &other) const noexcept
 	{
 		return !(*this == other);
 	}
 
-	bool operator<(const const_iterator &other) const throw()
+	bool operator<(const const_iterator &other) const noexcept
 	{
 		return i < other.i;
 	}
 
-	bool operator>(const const_iterator &other) const throw()
+	bool operator>(const const_iterator &other) const noexcept
 	{
 		return other < *this;
 	}
 
-	bool operator<=(const const_iterator &other) const throw()
+	bool operator<=(const const_iterator &other) const noexcept
 	{
 		return !(other < *this);
 	}
 
-	bool operator>=(const const_iterator &other) const throw()
+	bool operator>=(const const_iterator &other) const noexcept
 	{
 		return !(*this < other);
 	}
@@ -478,12 +496,12 @@ namespace internal {
 struct _iter_rep {
 	_iter_rep(const ex &e_, size_t i_, size_t i_end_) : e(e_), i(i_), i_end(i_end_) {}
 
-	bool operator==(const _iter_rep &other) const throw()
+	bool operator==(const _iter_rep &other) const noexcept
 	{
 		return are_ex_trivially_equal(e, other.e) && i == other.i;
 	}
 
-	bool operator!=(const _iter_rep &other) const throw()
+	bool operator!=(const _iter_rep &other) const noexcept
 	{
 		return !(*this == other);
 	}
@@ -497,7 +515,7 @@ struct _iter_rep {
 
 class const_preorder_iterator : public std::iterator<std::forward_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
 public:
-	const_preorder_iterator() throw() {}
+	const_preorder_iterator() noexcept {}
 
 	const_preorder_iterator(const ex &e, size_t n)
 	{
@@ -528,18 +546,18 @@ public:
 		return tmp;
 	}
 
-	bool operator==(const const_preorder_iterator &other) const throw()
+	bool operator==(const const_preorder_iterator &other) const noexcept
 	{
 		return s == other.s;
 	}
 
-	bool operator!=(const const_preorder_iterator &other) const throw()
+	bool operator!=(const const_preorder_iterator &other) const noexcept
 	{
 		return !(*this == other);
 	}
 
 private:
-	std::stack<internal::_iter_rep, std::vector<internal::_iter_rep> > s;
+	std::stack<internal::_iter_rep, std::vector<internal::_iter_rep>> s;
 
 	void increment()
 	{
@@ -561,7 +579,7 @@ private:
 
 class const_postorder_iterator : public std::iterator<std::forward_iterator_tag, ex, ptrdiff_t, const ex *, const ex &> {
 public:
-	const_postorder_iterator() throw() {}
+	const_postorder_iterator() noexcept {}
 
 	const_postorder_iterator(const ex &e, size_t n)
 	{
@@ -593,18 +611,18 @@ public:
 		return tmp;
 	}
 
-	bool operator==(const const_postorder_iterator &other) const throw()
+	bool operator==(const const_postorder_iterator &other) const noexcept
 	{
 		return s == other.s;
 	}
 
-	bool operator!=(const const_postorder_iterator &other) const throw()
+	bool operator!=(const const_postorder_iterator &other) const noexcept
 	{
 		return !(*this == other);
 	}
 
 private:
-	std::stack<internal::_iter_rep, std::vector<internal::_iter_rep> > s;
+	std::stack<internal::_iter_rep, std::vector<internal::_iter_rep>> s;
 
 	void descend()
 	{
@@ -626,12 +644,12 @@ private:
 	}
 };
 
-inline const_iterator ex::begin() const throw()
+inline const_iterator ex::begin() const noexcept
 {
 	return const_iterator(*this, 0);
 }
 
-inline const_iterator ex::end() const throw()
+inline const_iterator ex::end() const noexcept
 {
 	return const_iterator(*this, nops());
 }
@@ -641,7 +659,7 @@ inline const_preorder_iterator ex::preorder_begin() const
 	return const_preorder_iterator(*this, nops());
 }
 
-inline const_preorder_iterator ex::preorder_end() const throw()
+inline const_preorder_iterator ex::preorder_end() const noexcept
 {
 	return const_preorder_iterator();
 }
@@ -651,7 +669,7 @@ inline const_postorder_iterator ex::postorder_begin() const
 	return const_postorder_iterator(*this, nops());
 }
 
-inline const_postorder_iterator ex::postorder_end() const throw()
+inline const_postorder_iterator ex::postorder_end() const noexcept
 {
 	return const_postorder_iterator();
 }
@@ -669,19 +687,19 @@ inline bool are_ex_trivially_equal(const ex &e1, const ex &e2)
 }
 
 /* Function objects for STL sort() etc. */
-struct ex_is_less : public std::binary_function<ex, ex, bool> {
+struct ex_is_less {
 	bool operator() (const ex &lh, const ex &rh) const { return lh.compare(rh) < 0; }
 };
 
-struct ex_is_equal : public std::binary_function<ex, ex, bool> {
+struct ex_is_equal {
 	bool operator() (const ex &lh, const ex &rh) const { return lh.is_equal(rh); }
 };
 
-struct op0_is_equal : public std::binary_function<ex, ex, bool> {
+struct op0_is_equal {
 	bool operator() (const ex &lh, const ex &rh) const { return lh.op(0).is_equal(rh.op(0)); }
 };
 
-struct ex_swap : public std::binary_function<ex, ex, void> {
+struct ex_swap {
 	void operator() (ex &lh, ex &rh) const { lh.swap(rh); }
 };
 
@@ -709,7 +727,7 @@ inline ex imag_part(const ex & thisex)
 inline bool has(const ex & thisex, const ex & pattern, unsigned options = 0)
 { return thisex.has(pattern, options); }
 
-inline bool find(const ex & thisex, const ex & pattern, lst & found)
+inline bool find(const ex & thisex, const ex & pattern, exset& found)
 { return thisex.find(pattern, found); }
 
 inline bool is_polynomial(const ex & thisex, const ex & vars)
@@ -733,11 +751,8 @@ inline ex denom(const ex & thisex)
 inline ex numer_denom(const ex & thisex)
 { return thisex.numer_denom(); }
 
-inline ex normal(const ex & thisex, int level=0)
-{ return thisex.normal(level); }
-
-inline ex to_rational(const ex & thisex, lst & repl_lst)
-{ return thisex.to_rational(repl_lst); }
+inline ex normal(const ex & thisex)
+{ return thisex.normal(); }
 
 inline ex to_rational(const ex & thisex, exmap & repl)
 { return thisex.to_rational(repl); }
@@ -745,17 +760,14 @@ inline ex to_rational(const ex & thisex, exmap & repl)
 inline ex to_polynomial(const ex & thisex, exmap & repl)
 { return thisex.to_polynomial(repl); }
 
-inline ex to_polynomial(const ex & thisex, lst & repl_lst)
-{ return thisex.to_polynomial(repl_lst); }
-
 inline ex collect(const ex & thisex, const ex & s, bool distributed = false)
 { return thisex.collect(s, distributed); }
 
-inline ex eval(const ex & thisex, int level = 0)
-{ return thisex.eval(level); }
+inline ex eval(const ex & thisex)
+{ return thisex.eval(); }
 
-inline ex evalf(const ex & thisex, int level = 0)
-{ return thisex.evalf(level); }
+inline ex evalf(const ex & thisex)
+{ return thisex.evalf(); }
 
 inline ex evalm(const ex & thisex)
 { return thisex.evalm(); }
@@ -769,7 +781,7 @@ inline ex diff(const ex & thisex, const symbol & s, unsigned nth = 1)
 inline ex series(const ex & thisex, const ex & r, int order, unsigned options = 0)
 { return thisex.series(r, order, options); }
 
-inline bool match(const ex & thisex, const ex & pattern, lst & repl_lst)
+inline bool match(const ex & thisex, const ex & pattern, exmap& repl_lst)
 { return thisex.match(pattern, repl_lst); }
 
 inline ex simplify_indexed(const ex & thisex, unsigned options = 0)
@@ -832,7 +844,7 @@ protected:
 	ex (*ptr)(const ex &);
 public:
 	explicit pointer_to_map_function(ex x(const ex &)) : ptr(x) {}
-	ex operator()(const ex & e) { return ptr(e); }
+	ex operator()(const ex & e) override { return ptr(e); }
 };
 
 template<class T1>
@@ -842,7 +854,7 @@ protected:
 	T1 arg1;
 public:
 	explicit pointer_to_map_function_1arg(ex x(const ex &, T1), T1 a1) : ptr(x), arg1(a1) {}
-	ex operator()(const ex & e) { return ptr(e, arg1); }
+	ex operator()(const ex & e) override { return ptr(e, arg1); }
 };
 
 template<class T1, class T2>
@@ -853,7 +865,7 @@ protected:
 	T2 arg2;
 public:
 	explicit pointer_to_map_function_2args(ex x(const ex &, T1, T2), T1 a1, T2 a2) : ptr(x), arg1(a1), arg2(a2) {}
-	ex operator()(const ex & e) { return ptr(e, arg1, arg2); }
+	ex operator()(const ex & e) override { return ptr(e, arg1, arg2); }
 };
 
 template<class T1, class T2, class T3>
@@ -865,7 +877,7 @@ protected:
 	T3 arg3;
 public:
 	explicit pointer_to_map_function_3args(ex x(const ex &, T1, T2, T3), T1 a1, T2 a2, T3 a3) : ptr(x), arg1(a1), arg2(a2), arg3(a3) {}
-	ex operator()(const ex & e) { return ptr(e, arg1, arg2, arg3); }
+	ex operator()(const ex & e) override { return ptr(e, arg1, arg2, arg3); }
 };
 
 template<class C>
@@ -875,7 +887,7 @@ protected:
 	C &c;
 public:
 	explicit pointer_to_member_to_map_function(ex (C::*member)(const ex &), C &obj) : ptr(member), c(obj) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e); }
 };
 
 template<class C, class T1>
@@ -886,7 +898,7 @@ protected:
 	T1 arg1;
 public:
 	explicit pointer_to_member_to_map_function_1arg(ex (C::*member)(const ex &, T1), C &obj, T1 a1) : ptr(member), c(obj), arg1(a1) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1); }
 };
 
 template<class C, class T1, class T2>
@@ -898,7 +910,7 @@ protected:
 	T2 arg2;
 public:
 	explicit pointer_to_member_to_map_function_2args(ex (C::*member)(const ex&, T1, T2), C &obj, T1 a1, T2 a2) : ptr(member), c(obj), arg1(a1), arg2(a2) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1, arg2); }
 };
 
 template<class C, class T1, class T2, class T3>
@@ -911,7 +923,7 @@ protected:
 	T3 arg3;
 public:
 	explicit pointer_to_member_to_map_function_3args(ex (C::*member)(const ex &, T1, T2, T3), C &obj, T1 a1, T2 a2, T3 a3) : ptr(member), c(obj), arg1(a1), arg2(a2), arg3(a3) {}
-	ex operator()(const ex & e) { return (c.*ptr)(e, arg1, arg2, arg3); }
+	ex operator()(const ex & e) override { return (c.*ptr)(e, arg1, arg2, arg3); }
 };
 
 inline ex ex::map(ex f(const ex &)) const
@@ -966,20 +978,26 @@ inline void swap(GiNaC::ex &a, GiNaC::ex &b)
 	a.swap(b);
 }
 
-/** Specialization of std::iter_swap() for vector<ex> iterators. */
-template <>
-inline void iter_swap(vector<GiNaC::ex>::iterator i1, vector<GiNaC::ex>::iterator i2)
+/** Specialization of std::hash() for ex objects. */
+template<>
+struct hash<GiNaC::ex>
 {
-	i1->swap(*i2);
-}
+	std::size_t operator()(const GiNaC::ex & e) const noexcept
+	{
+		return e.gethash();
+	}
+};
 
-/** Specialization of std::iter_swap() for list<ex> iterators. */
-template <>
-inline void iter_swap(list<GiNaC::ex>::iterator i1, list<GiNaC::ex>::iterator i2)
+/** Specialization of std::equal_to() for ex objects. */
+template<>
+struct equal_to<GiNaC::ex>
 {
-	i1->swap(*i2);
-}
+	bool operator()(const GiNaC::ex &e1, const GiNaC::ex &e2) const noexcept
+	{
+		return e1.is_equal(e2);
+	}
+};
 
 } // namespace std
 
-#endif // ndef __GINAC_EX_H__
+#endif // ndef GINAC_EX_H
