@@ -3,7 +3,7 @@
  *  Interface to sequences of expression pairs. */
 
 /*
- *  GiNaC Copyright (C) 1999-2015 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2022 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -26,29 +26,12 @@
 #include "expair.h"
 #include "indexed.h"
 
-// CINT needs <algorithm> to work properly with <vector> and <list>
-#include <algorithm>
-#include <list>
-#include <memory>
 #include <vector>
 
 namespace GiNaC {
 
-/** Using hash tables can potentially enhance the asymptotic behaviour of
- *  combining n terms into one large sum (or n terms into one large product)
- *  from O(n*log(n)) to about O(n).  There are, however, several drawbacks.
- *  The constant in front of O(n) is quite large, when copying such an object
- *  one also has to copy the has table, comparison is quite expensive because
- *  there is no ordering any more, it doesn't help at all when combining two
- *  expairseqs because due to the presorted nature the behaviour would be
- *  O(n) anyways, the code is quite messy, etc, etc.  The code is here as
- *  an example for following generations to tinker with. */
-#define EXPAIRSEQ_USE_HASHTAB 0
-
 typedef std::vector<expair> epvector;       ///< expair-vector
 typedef epvector::iterator epp;             ///< expair-vector pointer
-typedef std::list<epp> epplist;             ///< list of expair-vector pointers
-typedef std::vector<epplist> epplistvector; ///< vector of epplist
 
 /** Complex conjugate every element of an epvector. Returns zero if this
  *  does not change anything. */
@@ -72,34 +55,34 @@ public:
 	expairseq(const ex & lh, const ex & rh);
 	expairseq(const exvector & v);
 	expairseq(const epvector & v, const ex & oc, bool do_index_renaming = false);
-	expairseq(std::auto_ptr<epvector>, const ex & oc, bool do_index_renaming = false);
+	expairseq(epvector && vp, const ex & oc, bool do_index_renaming = false);
 	
 	// functions overriding virtual functions from base classes
 public:
-	unsigned precedence() const {return 10;}
-	bool info(unsigned inf) const;
-	size_t nops() const;
-	ex op(size_t i) const;
-	ex map(map_function & f) const;
-	ex eval(int level=0) const;
-	ex to_rational(exmap & repl) const;
-	ex to_polynomial(exmap & repl) const;
-	bool match(const ex & pattern, exmap& repl_lst) const;
-	ex subs(const exmap & m, unsigned options = 0) const;
-	ex conjugate() const;
+	unsigned precedence() const override {return 10;}
+	bool info(unsigned inf) const override;
+	size_t nops() const override;
+	ex op(size_t i) const override;
+	ex map(map_function & f) const override;
+	ex eval() const override;
+	ex to_rational(exmap & repl) const override;
+	ex to_polynomial(exmap & repl) const override;
+	bool match(const ex & pattern, exmap& repl_lst) const override;
+	ex subs(const exmap & m, unsigned options = 0) const override;
+	ex conjugate() const override;
 
-	void archive(archive_node& n) const;
-	void read_archive(const archive_node& n, lst& syms);
+	void archive(archive_node& n) const override;
+	void read_archive(const archive_node& n, lst& syms) override;
 protected:
-	bool is_equal_same_type(const basic & other) const;
-	unsigned return_type() const;
-	unsigned calchash() const;
-	ex expand(unsigned options=0) const;
+	bool is_equal_same_type(const basic & other) const override;
+	unsigned return_type() const override;
+	unsigned calchash() const override;
+	ex expand(unsigned options=0) const override;
 	
 	// new virtual functions which can be overridden by derived classes
 protected:
 	virtual ex thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming = false) const;
-	virtual ex thisexpairseq(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming = false) const;
+	virtual ex thisexpairseq(epvector && vp, const ex & oc, bool do_index_renaming = false) const;
 	virtual void printseq(const print_context & c, char delim,
 	                      unsigned this_precedence,
 	                      unsigned upper_precedence) const;
@@ -121,7 +104,6 @@ protected:
 protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_tree(const print_tree & c, unsigned level) const;
-	void construct_from_2_ex_via_exvector(const ex & lh, const ex & rh);
 	void construct_from_2_ex(const ex & lh, const ex & rh);
 	void construct_from_2_expairseq(const expairseq & s1,
 	                                const expairseq & s2);
@@ -129,55 +111,27 @@ protected:
 	                                 const ex & e);
 	void construct_from_exvector(const exvector & v);
 	void construct_from_epvector(const epvector & v, bool do_index_renaming = false);
+	void construct_from_epvector(epvector && v, bool do_index_renaming = false);
 	void make_flat(const exvector & v);
 	void make_flat(const epvector & v, bool do_index_renaming = false);
 	void canonicalize();
 	void combine_same_terms_sorted_seq();
-#if EXPAIRSEQ_USE_HASHTAB
-	void combine_same_terms();
-	unsigned calc_hashtabsize(unsigned sz) const;
-	unsigned calc_hashindex(const ex & e) const;
-	void shrink_hashtab();
-	void remove_hashtab_entry(epvector::const_iterator element);
-	void move_hashtab_entry(epvector::const_iterator oldpos,
-	                        epvector::iterator newpos);
-	void sorted_insert(epplist & eppl, epvector::const_iterator elem);
-	void build_hashtab_and_combine(epvector::iterator & first_numeric,
-	                               epvector::iterator & last_non_zero,
-	                               vector<bool> & touched,
-	                               unsigned & number_of_zeroes);
-	void drop_coeff_0_terms(epvector::iterator & first_numeric,
-	                        epvector::iterator & last_non_zero,
-	                        vector<bool> & touched,
-	                        unsigned & number_of_zeroes);
-	bool has_coeff_0() const;
-	void add_numerics_to_hashtab(epvector::iterator first_numeric,
-	                             epvector::const_iterator last_non_zero);
-#endif // EXPAIRSEQ_USE_HASHTAB
 	bool is_canonical() const;
-	std::auto_ptr<epvector> expandchildren(unsigned options) const;
-	std::auto_ptr<epvector> evalchildren(int level) const;
-	std::auto_ptr<epvector> subschildren(const exmap & m, unsigned options = 0) const;
+	epvector expandchildren(unsigned options) const;
+	epvector evalchildren() const;
+	epvector subschildren(const exmap & m, unsigned options = 0) const;
 	
 // member variables
 	
 protected:
 	epvector seq;
 	ex overall_coeff;
-#if EXPAIRSEQ_USE_HASHTAB
-	epplistvector hashtab;
-	unsigned hashtabsize;
-	unsigned hashmask;
-	static unsigned maxhashtabsize;
-	static unsigned minhashtabsize;
-	static unsigned hashtabfactor;
-#endif // EXPAIRSEQ_USE_HASHTAB
 };
 
 /** Class to handle the renaming of dummy indices. It holds a vector of
- *  indices that are being used in the expression so-far. If the same
+ *  indices that are being used in the expression so far. If the same
  *  index occurs again as a dummy index in a factor, it is to be renamed.
- *  Unless dummy index renaming was swichted of, of course ;-) . */
+ *  Unless dummy index renaming was switched off, of course ;-) . */
 class make_flat_inserter
 {
 	public:
@@ -211,7 +165,7 @@ class make_flat_inserter
 				return x;
 			sort(dummies_of_factor.begin(), dummies_of_factor.end(), ex_is_less());
 			ex new_factor = rename_dummy_indices_uniquely(used_indices,
-				dummies_of_factor, x);
+			                                              dummies_of_factor, x);
 			combine_indices(dummies_of_factor);
 			return new_factor;
 		}
@@ -220,8 +174,8 @@ class make_flat_inserter
 		{
 			exvector new_dummy_indices;
 			set_union(used_indices.begin(), used_indices.end(),
-				dummies_of_factor.begin(), dummies_of_factor.end(),
-				std::back_insert_iterator<exvector>(new_dummy_indices), ex_is_less());
+			          dummies_of_factor.begin(), dummies_of_factor.end(),
+			          std::back_insert_iterator<exvector>(new_dummy_indices), ex_is_less());
 			used_indices.swap(new_dummy_indices);
 		}
 		bool do_renaming;
