@@ -6,7 +6,7 @@
  *  computation, square-free factorization and rational function normalization. */
 
 /*
- *  GiNaC Copyright (C) 1999-2020 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2022 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -2072,7 +2072,7 @@ static ex replace_with_symbol(const ex & e, exmap & repl, exmap & rev_lookup, ls
 	// they can be rationalised more efficiently
 	if (is_a<function>(e_replaced) && is_ex_the_function(e_replaced, exp)) {
 		for (auto & it : repl) {
-			if (is_a<function>(it.second) && is_ex_the_function(e_replaced, exp)) {
+			if (is_a<function>(it.second) && is_ex_the_function(it.second, exp)) {
 				ex ratio = normal(e_replaced.op(0) / it.second.op(0));
 				if (is_a<numeric>(ratio) && ex_to<numeric>(ratio).is_rational()) {
 					// Different exponents can be treated as powers of the same basic equation
@@ -2196,14 +2196,18 @@ ex basic::normal(exmap & repl, exmap & rev_lookup, lst & modifier) const
 
 	normal_map_function map_normal;
 	int nmod = modifier.nops(); // To watch new modifiers to the replacement list
-	lst result = dynallocate<lst>({replace_with_symbol(map(map_normal), repl, rev_lookup, modifier), _ex1});
+	ex result = replace_with_symbol(map(map_normal), repl, rev_lookup, modifier);
 	for (int imod = nmod; imod < modifier.nops(); ++imod) {
 		exmap this_repl;
 		this_repl.insert(std::make_pair(modifier.op(imod).op(0), modifier.op(imod).op(1)));
-		result = ex_to<lst>(result.subs(this_repl, subs_options::no_pattern));
+		result = result.subs(this_repl, subs_options::no_pattern);
 	}
 
-	return result;
+	// Sometimes we may obtain negative powers, they need to be placed to denominator
+	if (is_a<power>(result) && result.op(1).info(info_flags::negative))
+		return dynallocate<lst>({_ex1, power(result.op(0), -result.op(1))});
+	else
+		return dynallocate<lst>({result, _ex1});
 }
 
 
