@@ -3,7 +3,7 @@
  *  Interface to GiNaC's symmetry definitions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2022 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,16 +20,16 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef __GINAC_SYMMETRY_H__
-#define __GINAC_SYMMETRY_H__
+#ifndef GINAC_SYMMETRY_H
+#define GINAC_SYMMETRY_H
+
+#include "ex.h"
+#include "archive.h"
 
 #include <set>
 
-#include "ex.h"
-
 namespace GiNaC {
 
-
 class sy_is_less;
 class sy_swap;
 
@@ -61,6 +61,15 @@ public:
 	/** Create node with two children. */
 	symmetry(symmetry_type t, const symmetry &c1, const symmetry &c2);
 
+	// functions overriding virtual functions from base classes
+public:
+	/** Save (a.k.a. serialize) object into archive. */
+	void archive(archive_node& n) const override;
+	/** Read (a.k.a. deserialize) object from archive. */
+	void read_archive(const archive_node& n, lst& syms) override;
+protected:
+	unsigned calchash() const override;
+
 	// non-virtual functions in this class
 public:
 	/** Get symmetry type. */
@@ -80,7 +89,10 @@ public:
 
 	/** Check whether this node actually represents any kind of symmetry. */
 	bool has_symmetry() const {return type != none || !children.empty(); }
-
+	/** Check whether this node involves anything non symmetric. */
+	bool has_nonsymmetric() const;
+	/** Check whether this node involves a cyclic symmetry. */
+	bool has_cyclic() const;
 protected:
 	void do_print(const print_context & c, unsigned level) const;
 	void do_print_tree(const print_tree & c, unsigned level) const;
@@ -96,6 +108,7 @@ private:
 	/** Vector of child nodes. */
 	exvector children;
 };
+GINAC_DECLARE_UNARCHIVER(symmetry); 
 
 
 // global functions
@@ -136,7 +149,7 @@ const symmetry & antisymmetric4();
  *  @param v Start of expression vector
  *  @param symm Root node of symmetry tree
  *  @return the overall sign introduced by the reordering (+1, -1 or 0)
- *          or INT_MAX if nothing changed */
+ *          or numeric_limits<int>::max() if nothing changed */
 extern int canonicalize(exvector::iterator v, const symmetry &symm);
 
 /** Symmetrize expression over a set of objects (symbols, indices). */
@@ -157,7 +170,7 @@ inline ex antisymmetrize(const ex & e, const exvector & v)
 	return antisymmetrize(e, v.begin(), v.end());
 }
 
-/** Symmetrize expression by cyclic permuation over a set of objects
+/** Symmetrize expression by cyclic permutation over a set of objects
  *  (symbols, indices). */
 ex symmetrize_cyclic(const ex & e, exvector::const_iterator first, exvector::const_iterator last);
 
@@ -168,14 +181,6 @@ inline ex symmetrize_cyclic(const ex & e, const exvector & v)
 	return symmetrize(e, v.begin(), v.end());
 }
 
-// utility functions
-
-/** Specialization of is_exactly_a<symmetry>(obj) for symmetry objects. */
-template<> inline bool is_exactly_a<symmetry>(const basic & obj)
-{
-	return obj.tinfo()==TINFO_symmetry;
-}
-
 } // namespace GiNaC
 
-#endif // ndef __GINAC_SYMMETRY_H__
+#endif // ndef GINAC_SYMMETRY_H
